/*
  Copyright 1996, 1997, 2002 Ronald S. Burkey.

  This file is part of Ron's Indexing Program (RIP).

  RIP is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  RIP is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with GutenMark; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

  Filename:	rip.h 
  Target:	MS-DOS
  Compiler:	Borland Turbo C 2.0
  Purpose:	Header for rip.c and friends.
  Mods:		10/11/96 RSB	Started.
		03/07/97 RSB	RIP_TYPE added.
		01/08/02 RSB	Reformatted in GNU style and otherwise
				cleaned up in preparation for first
				GPL'd release.
*/

#include <stdlib.h>
#include <stdio.h>
#include <alloc.h>
#include <ctype.h>
#include <io.h>
#include <string.h>
#include <conio.h>
#include <time.h>
#include <dir.h>
#include <dos.h>
#include "fnkeys.h"

typedef long HASHVALUE;

#define ISVALID(x) (!isspace(x))
#define BLOCKSIZE 4096L

int read_huffman (char *filename, int c_index);
int iswimp (char *s);
HASHVALUE hash (char *s);
int rip (void);
int read_header (void);
int create_file (char *fil);
int open_file (char *fil);
int compress_it (char *infile, char *outfile);
int decompress_it (char *infile, char *outfile);
int index_it (void);
int ispunctuation (char c);
int longsort4 (int (*cmp) (void *, void *),
	       char *infile,
	       unsigned recsize,
	       unsigned long start, unsigned long end, char *outfile);

#define OVERBLOCK (BLOCKSIZE+256)	/* block size for indexing */
#define MINSTRING 4		/* min size of non-trash words */

/*
  The Huffman code tables used for compression and for decompression are
  in different formats.  For compression, for each possible character
  (0 to 255) we simply need to know how many bits are used, and what the
  bits are.  There are always less than 32 bits, so we can use a long
  to store the bits.  For decompression, since a variable number of bits
  is used, we need to make a binary tree.
*/
typedef struct
{
  unsigned numbits;
  unsigned long bits;
}
COMPRESS;
typedef COMPRESS COMPRESSOR[256];

/* "zero" is the offset in the array storing the tree of the zero branch.
   if zero==0, then there is none.  Similarly, "one" is the offset of the
   one branch, and one=0 if there isn't one.  If both are zero, then
   "value" is the decoded byte.  Otherwise, "value" is undefined. */
typedef struct
{
  char value;
  unsigned zero;
  unsigned one;
}
DECOMPRESS;
/* MAXD is the maximum number of nodes in the tree.  It might seem as though
   256 is enough, but since each atomic node must have a non-atomic node
   pointing to it, 256 cannot be enough.  On the other hand, each non-atomic
   node must actually point to two other nodes (atomic or non-atomic), rather
   than just one, since otherwise the tree wouldn't be minimal.  In fact
   it is easy to see by induction that any minimal tree with N atomic nodes
   must have N-1 non-atomic nodes. */
#define MAXD 511
typedef DECOMPRESS DECOMPRESSOR[MAXD];
#define MAX_SEARCH 16

typedef struct
{
  HASHVALUE hash;
  unsigned long offset;
}
NDX;

#define NDX_BUFSIZE 50
#define SEARCHSTRING_SIZE 129
typedef struct
{
  HASHVALUE hash;		/* =0 for wimps */
  unsigned long start_offset, end_offset;
  char string[SEARCHSTRING_SIZE];
}
SEARCH;

#define MAX_BOOKMARKS 10
#define MAX_NAMELENGTH 62
typedef struct
{
  unsigned long offset;
  char filename[MAX_NAMELENGTH];
}
BOOKMARK;

#define MAX_SEARCHWORDS 10

#define MAX_FILESFOUND 2000
typedef struct
{
  char indexno;
  unsigned fileno;
}
FOUND;

#define MAX_INDICES 100
#define MAX_INDEXNAME 21
#define MAX_INDEXTRAN 11
typedef struct
{
  char star2;
  char name[MAX_INDEXNAME];
  char src[MAX_INDEXTRAN];
  char dst[MAX_INDEXTRAN];
}
INDEXFILE;

/* ORIGINAL_RIP distinguishes between the source file rip.c and the other
   source files of the project. */
#ifdef ORIGINAL_RIP
#define RIP_TYPE
#else
#define RIP_TYPE extern
#endif
RIP_TYPE DECOMPRESSOR *decompressors[29];
RIP_TYPE COMPRESSOR *compressors[29];
RIP_TYPE FILE *fout, *fin, *ftemp, *ftemp2;
RIP_TYPE char s[1000];
RIP_TYPE BOOKMARK bookmarks[MAX_BOOKMARKS];
RIP_TYPE unsigned num_bookmarks;
RIP_TYPE unsigned long num_text, blocksize, index, blocks;
RIP_TYPE union
{
  char cb[0x2000];
  unsigned ib[0x1000];
}
file_hashes;
#define file_block file_hashes.cb
RIP_TYPE unsigned table_entries;
RIP_TYPE char text_buf[OVERBLOCK + 1];
RIP_TYPE int delete_file;
RIP_TYPE char outfile[129], infile[129];
RIP_TYPE SEARCH search[MAX_SEARCH];
RIP_TYPE unsigned num_search;
RIP_TYPE char searchwords[MAX_SEARCHWORDS][SEARCHSTRING_SIZE];
RIP_TYPE int num_searchwords;
unsigned findhash (FILE * f);
unsigned nexthash (void);
RIP_TYPE FOUND filesfound[MAX_FILESFOUND];
RIP_TYPE unsigned num_filesfound;
RIP_TYPE INDEXFILE indices[MAX_INDICES];
RIP_TYPE unsigned num_indices;
#ifdef ORIGINAL_RIP
int CLR_COMMAND = LIGHTGREEN, CLR_TEXT = YELLOW, CLR_MENU = WHITE;
int BCK_COMMAND = BLACK, BCK_TEXT = BLACK, BCK_MENU = BLACK;
#else
extern int CLR_COMMAND, CLR_TEXT, CLR_MENU;
extern int BCK_COMMAND, BCK_TEXT, BCK_MENU;
#endif


